﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Analysis.SolutionCaching {
  [Item("SolutionCache Analyzer Algorithm", "An algorithm that runs analyzers on a solution cache.")]
  [Creatable("Algorithms")]
  [StorableClass]
  public sealed class SolutionCacheAnalyzerAlgorithm : EngineAlgorithm, IStorableContent {
    public string Filename { get; set; }
    private const string analyzerParameterString = "SolutionCache Analyzer";
    private const string solutionCacheParameterString = "SolutionCache";

    #region Problem Properties
    public override Type ProblemType {
      get { return typeof(ISolutionCache); }
    }
    public new ISolutionCache Problem {
      get { return (ISolutionCache)base.Problem; }
      set { base.Problem = value; }
    }
    #endregion

    #region Parameter Properties
    public ValueParameter<CheckedItemList<ISolutionCacheAnalyzer>> SolutionCacheAnalyzerParameter {
      get { return (ValueParameter<CheckedItemList<ISolutionCacheAnalyzer>>)Parameters[analyzerParameterString]; }
    }
    public ValueParameter<ISolutionCache> SolutionCacheParameter {
      get { return (ValueParameter<ISolutionCache>)Parameters[solutionCacheParameterString]; }
    }
    #endregion

    [StorableConstructor]
    private SolutionCacheAnalyzerAlgorithm(bool deserializing) : base(deserializing) { }

    private SolutionCacheAnalyzerAlgorithm(SolutionCacheAnalyzerAlgorithm original, Cloner cloner)
      : base(original, cloner) {
    }

    public SolutionCacheAnalyzerAlgorithm()
      : base() {
      Parameters.Add(new ValueParameter<CheckedItemList<ISolutionCacheAnalyzer>>(analyzerParameterString, "Analyzers that can be applied to solution caches.", new CheckedItemList<ISolutionCacheAnalyzer>()));
      Parameters.Add(new ValueParameter<ISolutionCache>(solutionCacheParameterString, "The solution cache."));

      OperatorGraph.InitialOperator = new ParallelOperatorProcessor();
      //((ParallelOperatorProcessor)OperatorGraph.InitialOperator).ParallelParameter.Value.Value = true;
    }


    public override IDeepCloneable Clone(Cloner cloner) {
      return new SolutionCacheAnalyzerAlgorithm(this, cloner);
    }

    public override void Start() {
      if (SolutionCacheAnalyzerParameter.Value.CheckedItems.Count() == 0)
        throw new ArgumentException("At least one analyzer has to be selected. ");

      var initialOperator = OperatorGraph.InitialOperator as ParallelOperatorProcessor;
      initialOperator.Operators.Clear();

      foreach (var curOpt in SolutionCacheAnalyzerParameter.Value.CheckedItems) {
        initialOperator.Operators.Add(curOpt.Value);
      }

      base.Start();
    }

    private void UpdateSolutionCacheAnalyzers() {
      //TODO: should we fetch the operators from the problem? 
      SolutionCacheAnalyzerParameter.Value.Clear();
      foreach (var analyzer in ApplicationManager.Manager.GetInstances<ISolutionCacheAnalyzer>()) {
        SolutionCacheAnalyzerParameter.Value.Add(analyzer);
      }
    }

    #region Events
    protected override void OnProblemChanged() {
      UpdateSolutionCacheAnalyzers();
      SolutionCacheParameter.Value = Problem;
      base.OnProblemChanged();
    }
    #endregion
  }
}
